/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gAnnozillaSidebar;

function initialize() {

    gAnnozillaSidebar = new Annozilla();
    bundle = gAnnozillaSidebar.bundle;
}

function Annozilla() {

    // test for required components
    if (! Annozilla._hasRequiredComponents()) {
	return;
    }

    this.bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");

    // initialize annotations service
    this.annotService = Components.classes[Annozilla.ANNOTATIONS_SERVICE_CONTRACTID].getService().QueryInterface(Components.interfaces.nsIAnnotationsService);

    // get the annozilla AnnotationsApplication object
    this.app = this.annotService.registerApplication("annozilla");

    // add our observer
    this.app.addObserver(this);

    // prefs module
    this.prefs = new AnnozillaPrefs(this);

    // sidebar module
    this.sidebar = new AnnozillaSidebar(this.bundle);

    // icons module
    this.icons = window.parent.gIconManager;

    // inform icons module that we're running
    this.icons.sbarOnLoad();

    var that = this;

    // add a listener for when the window closes
    window.addEventListener("unload",
    			    function(e) { that.unload(); },
			    false);
}

Annozilla.UNKNOWN_REASON_URL = "about:blank";

Annozilla.ANNOTATIONS_SERVICE_CONTRACTID = "@mozilla.org/annotations_service;1";
Annozilla.XPOINTER_SERVICE_CONTRACTID = "@mozilla.org/xpointer-service;1";

Annozilla.PAGE_LOADING = 0;
Annozilla.PAGE_COMPLETED = 1;
Annozilla.PAGE_FAILED = 2;

Annozilla.ANNOT_NS = "http://www.w3.org/2000/10/annotation-ns#";
Annozilla.RDF_NS   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
Annozilla.DC10_NS    = "http://purl.org/dc/elements/1.0/";
Annozilla.DC11_NS    = "http://purl.org/dc/elements/1.1/";
Annozilla.THREAD_NS= "http://www.w3.org/2001/03/thread#";
Annozilla.XHTML_NS = "http://www.w3.org/1999/xhtml";
Annozilla.TYPE_NS = "http://www.w3.org/2000/10/annotationType#";

Annozilla.prototype = {
    annotations: undefined,
    bundle: undefined,
    userLoadInProgress: undefined,
    currentURI: undefined,
    currentURIState: undefined,
    serversResponded: undefined,
    serverCount: undefined
};

Annozilla.prototype.unload = function() {
    // remove observer
    this.app.removeObserver(this);

    // tell prefs we're shutting down
    this.prefs.destructor();
    
    // tell icons we're shutting down
    this.icons.sbarOnUnload();
}
    

Annozilla.prototype.QueryInterface = function(iid) {  
    if (! (iid.equals(Components.interfaces.nsIAnnotationsObserver) ||
	   iid.equals(Components.interfaces.nsISupports) ) ) {
	       throw Components.results.NS_ERROR_NO_INTERFACE;
	   }
    return this;
}
 
/**
 * Responds to a user pressing the load button
 * by loading annotations for the current page.
 * Signifies that from here on out, we should
 * pay attention to load callbacks instead of autoLoad
 * callbacks.
 */   
Annozilla.prototype.userLoadOnCommand = function() {
    this.userLoadInProgress = true;

    this.currentURI = Annozilla.stripURL(window._content.location.href);
    this.annotations = {};
    this.serversResponded = 0;
    this.serverCount = this.app.getServerCount();

    if (this.serverCount == 0) {
        this.sidebar.clearList();
        this.sidebar.setHeader (bundle.GetStringFromName("NO_SERVERS"));
        return;
    }

    this.sidebar.clearList();
    this.sidebar.setHeader(
        this.bundle.formatStringFromName (
            "USERLOADING_COUNT", [0, this.serverCount], 2));
    this.sidebar.operationInProgress("LOADING");
    this.icons.sbarUserLoadBegin(this.currentURI);

    // do the load
    this.app.loadAnnotations(this.currentURI);
}

Annozilla.prototype.postOnCommand = function() {

   /* Prefered test is to check whether the htmleditor class is installed;
      however on MozillaFirebird 0.6.1 this test returns true.
      Backup test is to test for the existence of the element "key_editPage"
      in the top browser window - this is associated with the "File ->
      Edit Page" menu option.
    */
    if (!("@mozilla.org/editor/htmleditor;1" in Components.classes) ||
        window.top.document.getElementById ("key_editPage") == undefined) {
        window.alert ("HTML editor not installed.\n(This functionality is not yet supported in Firefox.)");
        return;
    }
    // get the xptr
    var baseURL = Annozilla.stripURL(window._content.location.href);
    var xptr = xptrCreateXPointer(window._content.getSelection(),
				  window._content.document);

    window.openDialog("chrome://annozilla/content/edit_body.xul",
		      "annotation-body" + Math.random(),
		      "chrome,resizable,dialog=no",
		      null,
		      baseURL,
		      xptr,
		      null);
}

Annozilla.prototype.updateOnCommand = function() {

   /* Prefered test is to check whether the htmleditor class is installed;
      however on MozillaFirebird 0.6.1 this test returns true.
      Backup test is to test for the existence of the element "key_editPage"
      in the top browser window - this is associated with the "File ->
      Edit Page" menu option.
    */
    if (!("@mozilla.org/editor/htmleditor;1" in Components.classes) ||
        window.top.document.getElementById ("key_editPage") == undefined) {
        window.alert ("HTML editor not installed.\n(This functionality is not yet supported in Mozilla Firefox.)");
        return;
    }

    // get the selected item
    var annotationURI = this.sidebar.getSelectedAnnotationURI();
    if (! (annotationURI && this.annotations[annotationURI]) ) {
	window.alert("Unable to edit the selected annotation due to an internal error. " +
		     " Try reloading the annotations for the page.");
	return;
    }

    var annotation = this.annotations[annotationURI].annot.clone();

    // if this is a local annotation
    // we must generate a local url for its body
    if (annotation.serverName == this.app.LOCALSERVER_URI) {
      bodyURI = LocalServerUtils.createTempURIFromString(annotation.getBodyText());
    }
    else {
      bodyURI = annotation.bodyURI;
    }
  
    window.openDialog("chrome://annozilla/content/edit_body.xul",
		      "annotation-body" + Math.random(),
		      "chrome,resizable,dialog=no",
		      annotation,
		      null, null, bodyURI);
}

Annozilla.prototype.deleteOnCommand = function() {
    var annotationURI = this.sidebar.getSelectedAnnotationURI();
    if (! (annotationURI && this.annotations[annotationURI]) ) {
	window.alert("Unable to delete the selected annotation due to an internal error. " +
		     " Try reloading the annotations for the page.");
	return;
    }

    var annotation = this.annotations[annotationURI].annot;

    this.app.deleteAnnotation(annotation);
}


Annozilla.prototype.loadBodyOnCommand = function() {
  var  annotationURI = this.sidebar.getSelectedAnnotationURI();
  if (! (annotationURI && this.annotations[annotationURI]) ) {
    window.alert("Unable to load the selected annotation's body due to an internal error. " +
		 " Try reloading the annotations for the page.");
    return;
  }
  
  var annotation = this.annotations[annotationURI].annot;
  var bodyURI;

  // if this is a local annotation
  // we must generate a local url for its body
  if (annotation.serverName == this.app.LOCALSERVER_URI) {
      bodyURI = LocalServerUtils.createTempURIFromString(annotation.getBodyText());
  }
  else {
    bodyURI = annotation.bodyURI;
  }
  
  if (this.prefs.getPref("loadInNewWindow")) {
    window.open(bodyURI, "_blank","width=600,height=320,scrollbars=yes,resizable=yes","");
  }
  else {
    this.sidebar.loadAnnotationBody(bodyURI);
  }
}
  

/**
 * Called when a page begins loading in the 
 * browser if the application has 
 * autoLoad turned on.  
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url loading
 */
Annozilla.prototype.autoLoadOnBeginPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
	    //return; // commented out because it can abandon legitimate autoload operations
    }

    // TODO should this be in the service?
    if (this.prefs.getPref("autoLoadMulti") == "sites") {
        var allowSite = false;
        var sites = this.prefs.getPref("autoLoadSites").split(" ");
        for (var ix=0; ix<sites.length; ix++) {
            if (url.indexOf(sites[ix]) == 0) {
                allowSite = true;
                break;
            }
        }
        if (!allowSite) {
            return;
        }
    }

    this.currentURI = url;
    this.currentURIState = Annozilla.PAGE_LOADING;
    this.userLoadInProgress = false;
    this.annotations = {};

    this.serversResponded = 0;
    this.serverCount = this.app.getServerCount();

    if (this.serverCount == 0) {
        this.sidebar.clearList();
        this.sidebar.setHeader (bundle.GetStringFromName("NO_SERVERS"));
        return;
    }

    this.sidebar.clearList();
    this.sidebar.setHeader(
        this.bundle.formatStringFromName (
            "AUTOLOADING_COUNT", [0, this.serverCount], 2));
    this.sidebar.operationInProgress("AUTOLOADING");
}

/**
 * Called when a page successfully finishes loading.
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url finished loading
 * @param annotations nsISimpleEnumeration of annotations for url
 */
Annozilla.prototype.autoLoadOnSuccessPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
	return;
    }

    this.currentURIState = Annozilla.PAGE_COMPLETE;

    // set the taskbar
    this._setTaskBar();
              
    // resolve all outstanding annotations
    for(uri in this.annotations) {
	var annotationObj = this.annotations[uri];

	if (! annotationObj.annot.data) {
	    this._debugBadAnnotation(annotationObj.annot);
	    continue;
	}

	annotationObj.range = xptrResolveAnnotation(annotationObj.annot, window._content.document);
    }

    // add these annotations to the sidebar
    this.sidebar.addAnnotations(this.annotations);
    if (this.serversResponded && this.serverCount) {
        this.sidebar.setHeader(
            this.bundle.formatStringFromName (
                "AUTOLOADING_COUNT", [this.serversResponded, this.serverCount], 2));
    }

    // add these annotations to the icon manager
    this.icons.sbarLoadedAnnotations(this.annotations);
}

/**
 * Called when a page that was loading is
 * unable to load (or is cancelled by the user).
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url aborted
 * @param message string error message
 */
Annozilla.prototype.autoLoadOnFailPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
	return;
    }

    this.currentURIState = Annozilla.PAGE_FAILED;

    // add the annotations to the sidebar
    this.sidebar.addAnnotations(this.annotations);

    // document failed page load
    if (this.serversResponded == this.serverCount) {
      this.sidebar.operationComplete("PAGE_LOAD_ERROR");
    }
    else {
      this.sidebar.operationInProgress("PAGE_LOAD_ERROR");
    }
}


/**
 * Success callback function to return the results for an
 * automatic annotations load (autoLoad).
 * This function returns the annotations for a single server, designated by serverURI.  
 * It will be called once for each server that successfully responds.
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri that the annotations annotate
 * @param annotations nsISimpleEnumerator of nsIAnnotations for annotatedURI that
 *                    were on serverURI annotations server
 */
Annozilla.prototype.autoLoadAnnotationsForServer = function(serverURI, win, annotatedURI,
							    annotationsLength, annotations) {

    // ensure correct window
    if (win != window.content) {
	return;
    }

    // if there is a user load in progress, ignore these annotations
    if (this.userLoadInProgress) {
	dump("ignoring autoLoad annotations from " + serverURI + " about " + annotatedURI + " since userLoad in progress\n");
	return;
    }

    this.serversResponded += 1;

    this._processAnnotationsWithAutoLoadInfo(this.currentURIState, annotations);

    this._setTaskBar();

    if (this.serversResponded && this.serverCount) {
        this.sidebar.setHeader(
            this.bundle.formatStringFromName (
                "AUTOLOADING_COUNT", [this.serversResponded, this.serverCount], 2));
    }
}

/**
 * Callback function to indicate failure for an automatic
 * annotations load (autoLoad).
 * This function will be called once for each annotations server that
 * fails in the autoloading process. 
 *
 * @param serverURI string uri of the annotations server that failed
 * @param annotatedURI string uri for which annotations were requested
 * @param message string error message
 */ 
Annozilla.prototype.autoLoadFailureForServer = function(serverURI, win, annotatedURI, message) {

    // ensure correct window
    if (win != window.content) {
	return;
    }

  this.serversResponded += 1;
}


/**
 * Success callback function to return the results for a loadAnnotations request
 *  - OR - an automatic autoLoad request.  This function returns the 
 * annotations for one server, designated by serverURI.  
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri that the annotations annotate
 * @param annotations nsISimpleEnumerator of nsIAnnotations for annotatedURI that
 *                    were on serverURI annotations server
 */
Annozilla.prototype.loadAnnotationsOnSuccess = function(serverURI, annotatedURI, 
							annotationsLength, annotations) {
  // if we're autoLoading, ignore these annotations
  if (! this.userLoadInProgress) {
    dump("ignoring loadAnnotationsOnSuccess of " + annotatedURI + " from " + serverURI + 
	 " because userLoad not in progress\n");
    return;
  }
  
  // if we've moved on to a new page, ignore these annotations
  if (annotatedURI != this.currentURI) {
    dump("ignoring loadAnnotationsOnSuccess of " + annotatedURI + " from " + serverURI + 
	 " because it pertains to old URI.  currentURI: " + this.currentURI + "\n");
    return;
  }

  try {
  
      this.serversResponded += 1;
      
      // if autoLoad is enabled, we can benefit from its alerts about when the page is
      // fully loaded
      if (this._isAutoLoadEnabled()) {
        this._processAnnotationsWithAutoLoadInfo(this.currentURIState, annotations);
        
        this.sidebar.setHeader(
            this.bundle.formatStringFromName (
                "USERLOADING_COUNT", [this.serversResponded, this.serverCount], 2));
        this._setTaskBar();
      }
      // otherwise, we have to wing it. for now we'll assume that the page
      // is loaded, since there's no way that I know to check whether the DOM
      // is fully constructed
      else {
        var currentAnnotationsBatch = {};
        
        for (var i = 0; i < annotations.length; i++) {
          var current = annotations[i];

          // testing purposes
          if (! current.data) {
              this._debugBadAnnotation(current);
              continue;
          }

          var range = xptrResolveAnnotation(current, window._content.document);
          
          // add to the current batch
          Annozilla._addAnnotationToContainer(current, range, currentAnnotationsBatch);
          
          // add to the overall collection
          Annozilla._addAnnotationToContainer(current, range, this.annotations);
        }

        // add this batch to the sidebar
        this.sidebar.addAnnotations(currentAnnotationsBatch);
        
        if (this.serversResponded == this.serverCount) {
          this.sidebar.operationComplete("LOAD_OK");
        }
        this.sidebar.setHeader(
            this.bundle.formatStringFromName (
                "USERLOADING_COUNT", [this.serversResponded, this.serverCount], 2));
        
        // add this batch to the icon manager
        this.icons.sbarLoadedAnnotations(currentAnnotationsBatch);
      }

    } catch (e) {
      var console = Components.classes['@mozilla.org/consoleservice;1']
          .getService(Components.interfaces.nsIConsoleService);
      console.logStringMessage ("Annozilla: error loading annotations " + e);
      this.sidebar.operationComplete("LOAD_ERROR");
    }
}

/**
 * Callback function to indicate failure for a loadAnnotations request
 *  - OR - an automatic autoLoad request.  This function indicates failure
 * for one particular annotations server, designated by serverURI.
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri for which annotations were requested
 * @param message string error message
 */
Annozilla.prototype.loadAnnotationsOnFailure = function(serverURI, annotatedURI, message) {
    dump("Annozilla: loadAnnotationsOnFailure for server: " + serverURI + " w/ no annotations of " + annotatedURI +
	 " because of error: " + message + "\n");
}

/**
 * Called when an annotation post request succeeds.
 *
 * @param newAnnotation nsIAnnotation the newly created annotation
 */
Annozilla.prototype.postAnnotationOnSuccess = function(newAnnotation) {
    // if this annotation applies to a page we're no longer viewing
    if (newAnnotation.data.getProperty(Annozilla.ANNOT_NS, "annotates") != this.currentURI) {
	dump("posted annotation applies to page no longer being viewed\n");
	return;
    }

    var currentAnnotationsBatch = {};

    var range = xptrResolveAnnotation(newAnnotation, window._content.document);
	    
    // add to the current batch
    Annozilla._addAnnotationToContainer(newAnnotation, range, currentAnnotationsBatch);

    // add to the overall collection
    Annozilla._addAnnotationToContainer(newAnnotation, range, this.annotations);
       
    // add this batch to the sidebar
    this.sidebar.addAnnotations(currentAnnotationsBatch);
}

/**
 * Called when an annotation post fails.
 *
 * @param message string error message
 */
Annozilla.prototype.postAnnotationOnFailure = function(message) {}



/**
 * Called when an annotation update succeeds.
 *
 * @param updatedAnnotation nsIAnnotation that has been updated on server
 */
Annozilla.prototype.updateAnnotationOnSuccess = function(updatedAnnotation) {
    if (this.annotations && this.annotations[updatedAnnotation.annotationURI]) {
	var oldRange = this.annotations[updatedAnnotation.annotationURI].range;
	// update the old annotation
	this.annotations[updatedAnnotation.annotationURI] = { annot: updatedAnnotation,
							      range: oldRange };
	
	this.sidebar.updateAnnotation(updatedAnnotation, oldRange);
    }
}

/**
 * Called when an annotation update fails.
 *
 * @param url string url of the annotation that could not be updated
 * @param message string error message
 */
Annozilla.prototype.updateAnnotationOnFailure = function(url, message) {}



/**
 * Called when an annotation is successfully deleted.
 *
 * @param url string url of deleted annotation
 */
Annozilla.prototype.deleteAnnotationOnSuccess = function(url) {
  if (this.annotations && this.annotations[url]) {
    delete this.annotations[url];
  }
   
  this.sidebar.deleteAnnotation(url);
}

/**
 * Called when an annotation delete fails.
 *
 * @param url string url of the annotation that could not be deleted
 * @param message string error message
 */
Annozilla.prototype.deleteAnnotationOnFailure = function(url, message) {
    window.alert("Unable to delete annotation " + url + "\nThe server responded: " + message + "\n");
}

/**
 * Called when the replies to an annotation have been successfully
 * loaded.
 * 
 * @param annotation nsIAnnotation for which replies were requested
 */
Annozilla.prototype.getRepliesOnSuccess = function(annotation, repliesCount, replies) {}

/**
 * Called when getting the replies to an annotation fails.
 *
 * @param annotation nsIAnnotation for which replies were requested
 * @param message string error message
 */
Annozilla.prototype.getRepliesOnFailure = function(annotation, message) {}

/**
 * Called an annotation has successfully been replied to.
 * 
 * @param original nsIAnnotation that existed originally
 * @param reply nsIAnnotation reply created
 */
Annozilla.prototype.replyToAnnotationOnSuccess = function(original, reply) {}

/**
 * Called when replying to an annotation fails.
 *
 * @param original nsIAnnotation to which we attempted to reply
 * @param message string error message
 */
Annozilla.prototype.replyToAnnotationOnFailure = function(original, message) {}

/**
 * Called after a loadAnnotationBody request
 * when the body text of the annotation has been successfully loaded.
 *
 * @param annotation nsIAnnotation with updated body text
 * @param bodyText string text of the annotation's body
 */
Annozilla.prototype.loadAnnotationBodyOnSuccess = function(annotation, bodyText) {}

/**
 * Called after a loadAnnotationBody request when
 * the body text fails to load.
 *
 * @param annotation nsIAnnotation that body was requested for (unmodified)
 * @param message string error message
 */
Annozilla.prototype.loadAnnotationBodyOnFailure = function(annotation, message) {}

Annozilla._hasRequiredComponents = function() {
    if (! Components.classes[Annozilla.ANNOTATIONS_SERVICE_CONTRACTID]) {
	window.alert("The Annozilla sidebar requires the Annotations Service.  Please install it " +
		     "from http://annozilla.mozdev.org/installation/");
	return false;
    }
    if (! Components.classes[Annozilla.XPOINTER_SERVICE_CONTRACTID]) {
	window.alert("The Annozilla sidebar requires the XPointerService from XPointerLib. " +
		     "Please install it from http://xpointerlib.mozdev.org/installation.html");
	return false;
    }
    
    return true;
}

/**
 * Private function to strip query
 * strings off of URLs. (we don't want
 * to be nosy!)
 *
 * @param url String URL to strip query strings from
 * @return URL string stripped of queries
 */
Annozilla.stripURL = function (url) {
    if (!url || ('string' != typeof(url)) ||  ('' == url) || url == 'about:blank') {
	return Annozilla.UNKNOWN_REASON_URL;
    }
    
    // Strip off any query strings (Don't want to be too nosy).
    var queryPart = url.indexOf('?');
    if (queryPart != -1) {
	url = url.slice(0, queryPart);
    }
	
    return url;    
}

Annozilla.prototype._isAutoLoadEnabled = function() {
    return (this.prefs.getPref("autoLoadMulti") != "never");
}

Annozilla._addAnnotationToContainer = function(aAnnotation, aRange, aContainer) {
  aContainer[aAnnotation.annotationURI] = { annot: aAnnotation,
					    range: aRange  };
}

Annozilla.prototype._setTaskBar = function() {
  
    // if all servers are responded, update taskbar
    if (this.serversResponded == this.serverCount) {
      if (this.currentURIState == Annozilla.PAGE_LOADING) {
	this.sidebar.operationInProgress("LOAD_WAIT");
      }
      else if (this.currentURIState == Annozilla.PAGE_FAILED) {
	this.sidebar.operationComplete("PAGE_LOAD_ERROR");
      }
      else {
	this.sidebar.operationComplete("LOAD_OK");
      }
    }
}

Annozilla.prototype._debugBadAnnotation = function(annotation) {

    dump("annotation.data is NULL.  enumerating properties:\n");
    dump("serverName: " + annotation.serverName + "\n");
    dump("annotationURI: " + annotation.annotationURI + "\n");
    dump("bodyURI: " + annotation.bodyURI + "\n");
    dump("getBodyText: " + annotation.getBodyText() + "\n");
    dump("setting bodyText.\n");
    annotation.setBodyText("New test body text");
    dump("new body text: " + annotation.getBodyText() + "\n");
}

Annozilla.prototype._processAnnotationsWithAutoLoadInfo = function(state, annotations) {
   
    if (state == Annozilla.PAGE_LOADING) {
	// add the annotations without resolving their context
      for (var i = 0; i < annotations.length; i++) {
	  Annozilla._addAnnotationToContainer(annotations[i], null, this.annotations);
      }
    }
    else if (state == Annozilla.PAGE_COMPLETE) {
      var currentAnnotationsBatch = {};
      
      for (var i = 0; i < annotations.length; i++) {
	  var current = annotations[i];
	
	  if (! current.data) {
	      this._debugBadAnnotation(current);
	      continue;
	  }
	  
	  var range = xptrResolveAnnotation(current, window._content.document);
	  	  
	  // add to the current batch
	  Annozilla._addAnnotationToContainer(current, range, currentAnnotationsBatch);
	  
	  // add to the overall collection
	  Annozilla._addAnnotationToContainer(current, range, this.annotations);
      }
      
      // add this batch to the sidebar
      this.sidebar.addAnnotations(currentAnnotationsBatch);
      
      // add this batch to the icon manager
      this.icons.sbarLoadedAnnotations(currentAnnotationsBatch);
    }
    else {
      var currentAnnotationsBatch = {};
      
      for (var i = 0; i < annotations.length; i++) {
	var current = annotations[i];
	
	if (! current.data) {
	    this._debugBadAnnotation(current);
	    continue;
	}
	  
	// add to the current batch
	Annozilla._addAnnotationToContainer(current, null, currentAnnotationsBatch);
	
	// add to the overall collection
	Annozilla._addAnnotationToContainer(current, null, this.annotations);
      }

      // add this batch to the sidebar
      this.sidebar.addAnnotations(currentAnnotationsBatch);
      
//       // add this batch to the icon manager
//       this.icons.sbarLoadedAnnotations(currentAnnotationsBatch);
    }
    
    // otherwise, the page failed to load, so we ignore annotations
    
}
    	
Annozilla.prototype.getURLSpecFromFile = function(file) {
    
    if (Components.classes["@mozilla.org/network/io-service;1"] && Components.interfaces.nsIIOService) {
	
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);

	// new version
	if (fileHandler.getURLSpecFromFile) {
	    return fileHandler.getURLSpecFromFile(file);
	}
	// old version in ioService
	else {
	    return ioService.getURLSpecFromFile(file);
	}
    }
    // really old version
    else {
	var file_url = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIFileURL);
	file_url.file = file;
	return file_url.spec;
    }
}

function AnnozillaSidebar(bundle) {
    this.bundle = bundle;
}

AnnozillaSidebar.prototype.clearList = function() {
    clearList();
}

AnnozillaSidebar.prototype.operationInProgress = function(bundleStr) {
  operationInProgress(this.bundle.GetStringFromName(bundleStr));
}

AnnozillaSidebar.prototype.operationComplete = function(bundleStr) {
  operationComplete(this.bundle.GetStringFromName(bundleStr));
}

AnnozillaSidebar.prototype.setHeader = function(message) {
  setHeader(message);
}

AnnozillaSidebar.prototype.addAnnotations = function(annotations) {
  for (uri in annotations) {
      // XXX don't add broken annotations
      if (annotations[uri].annot.data) {
	  addNewAnnotation(annotations[uri].annot, annotations[uri].range, null);
      }
  }
}

AnnozillaSidebar.prototype.getSelectedAnnotationURI = function() {
    var tree = document.getElementById('annotationTree');
    var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
    if (selectedItem) {
	return selectedItem.getAttribute('-annozilla-resource');
    }
    else {
	return null;
    }
}

AnnozillaSidebar.prototype.updateAnnotation = function(annotation, range) {
    updateAnnotation(annotation);
}

AnnozillaSidebar.prototype.deleteAnnotation = function(url) {
    deleteAnnotationFromUI(url);
}

AnnozillaSidebar.prototype.loadAnnotationBody = function(bodyURI) {
  document.getElementById("body").loadURI(bodyURI);
}
